#include "BasicBlockWidget.hxx"
#include "gui/Mainwindow.hxx"

BasicBlockWidget::BasicBlockWidget(const QString& name, BasicBlock * block,
                                   Mainwindow * mainwindow)
	: width(270), height(45), name(name)
	, _widget("", this), _table(NULL)
	, block(block), mainwindow(mainwindow) {
	next[0] = NULL;	next[1] = NULL;
	_widget.setPos(5, 20);
	_widget.setTextInteractionFlags(Qt::TextSelectableByMouse|
	                                Qt::LinksAccessibleByMouse);

	if (width < 250) width = 250;
}

void BasicBlockWidget::addItem(uint8_t* bytes, size_t num_bytes,
                               QString line, const QString& href) {
	QString bytestring;
	int row;

	if (_table) {
		row = _table->rows();
		_table->appendRows(1);
	} else {
		row = 0;
		QTextTableFormat format;
		format.setBorderStyle(QTextFrameFormat::BorderStyle_None);
		format.setBorder(0);
		_table = _widget.textCursor().insertTable(1, 3, format);
	}

	for (size_t i(0); i < num_bytes; ++i) {
		const char * hexdigits = "0123456789ABCDEF";
		bytestring += hexdigits[(bytes[i] >> 4) & 0xF];
		bytestring += hexdigits[bytes[i] & 0xF];
		bytestring += ' ';
	}

	_table->cellAt(row, 0).firstCursorPosition().insertText(bytestring);

	line = line.replace('\t', ' ').toHtmlEscaped();
	if (href != "") {
		line = "<a href=\"" + href + "\">" + line + "</a>";
	}

	_table->cellAt(row, 1).firstCursorPosition().insertHtml(line);

	QObject::connect(&_widget, &QGraphicsTextItem::linkActivated,
	                 [=](QString str) {
		                 if (str.startsWith("function:")) {
			                 QString address = str.remove("function:");
			                 mainwindow->switchMainPlaneToAddress(address.toInt(NULL, 16));
		                 }
	                 });

	width = 10 + _widget.boundingRect().width();
	height = 25 + _widget.boundingRect().height();

	if (width < 250) width = 250;
}

void BasicBlockWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
	           QWidget *widget) {
	painter->fillRect(0, 0, width, height, QColor(0xcc, 0xcc, 0xff, 0xff));
	painter->setPen(QColor(0x00, 0x00, 0xff, 0xff));
	painter->drawRect(0, 0, width, height);
	painter->drawText(5, 15, name);
}

QRectF BasicBlockWidget::boundingRect() const  {
	qreal penWidth = 1;
	QRectF result(- penWidth / 2, - penWidth / 2,
	              width + penWidth, height + penWidth);
	return result;
}

std::array<QPointF, 3> BasicBlockWidget::getExits() const {
	return { { mapToScene(QPointF(  width/3, height)),
			   mapToScene(QPointF(  width/2, height)),
			   mapToScene(QPointF(2*width/3, height)) } };
}

